"use strict";

const path = require("path");
const log = require("../../utils/log");
const { exec: spawn } = require("../../utils/utils");

async function exec() {
  let targetPath = process.env.CLI_TARGET_PATH;
  const homePath = process.env.CLI_HOME_PATH;
  log.verbose(`targetPath: ${targetPath}`);
  log.verbose(`homePath: ${homePath}`);

  // current process exec
  // require("../init").call(null, Array.from(arguments));
  const rootFile = path.resolve(__dirname, "../init");
  if (rootFile) {
    try {
      const args = Array.from(arguments);
      const cmd = args[args.length - 1];
      // Handle valid data
      const obj = Object.create(null);
      Object.keys(cmd).forEach((key) => {
        if (
          cmd.hasOwnProperty(key) &&
          !key.startsWith("_") &&
          key !== "parent"
        ) {
          obj[key] = cmd[key];
        }
      });
      args[args.length - 1] = obj;
      const code = `require('${rootFile}').call(null, ${JSON.stringify(args)})`;
      // node child process exec
      const child = spawn("node", ["-e", code], {
        cwd: process.cwd(),
        stdio: "inherit",
      });
      child.on("exit", (e) => {
        // exit code 0 > success
        log.verbose("命令执行成功:" + e);
        process.exit(e);
      });
      child.on("error", (e) => {
        // exit code 1 > fail
        log.error(e.message);
        process.exit(1);
      });
    } catch (e) {
      log.error(e.message);
    }
  }
}

module.exports = exec;
